<?php
if(!defined('ABSPATH')) {die('You are not allowed to call this page directly.');}
/*
Integration of MailPoet into MemberPress
*/
class MpMailPoet  {
  public function __construct() {
    add_action('mepr_display_autoresponders',   array($this, 'display_option_fields'));
    add_action('mepr-process-options',          array($this, 'store_option_fields'));
    add_action('mepr-user-signup-fields',       array($this, 'display_signup_field'));
    add_action('mepr-signup',                   array($this, 'process_signup'));
    add_action('mepr-txn-store',                array($this, 'process_status_changes'));
    add_action('mepr-subscr-store',             array($this, 'process_status_changes'));
    add_action('mepr-txn-expired',              array($this, 'process_status_changes'), 10, 2);
    add_action('mepr-product-advanced-metabox', array($this, 'display_product_override'));
    add_action('mepr-product-save-meta',        array($this, 'save_product_override'));
    add_filter('mepr-save-account',             array($this, 'update_user'));

    // Enqueue scripts
    add_action('mepr-options-admin-enqueue-script', array($this, 'admin_enqueue_options_scripts'));
    add_action('mepr-product-admin-enqueue-script', array($this, 'admin_enqueue_product_scripts'));
  }

  public function admin_enqueue_options_scripts($hook) {
    wp_enqueue_script('mp-mailpoet-options-js', MPMAILPOET_URL.'/mailpoet_options.js');
    wp_localize_script('mp-mailpoet-options-js', 'MeprMailPoet', array('wpnonce' => wp_create_nonce(MEPR_PLUGIN_SLUG)));
  }

  public function admin_enqueue_product_scripts($hook) {
    wp_enqueue_script('mp-mailpoet-product-js', MPMAILPOET_URL.'/mailpoet_product.js');
  }

  public function update_user($mepr_user) {
    if(!$this->is_enabled()) { return; }

    //MailPoet does this for us, so we literally just have to call this static method
    WYSIJA::hook_edit_WP_subscriber($mepr_user->ID);
  }

  public function display_option_fields() {
    $all_lists = $this->get_lists();
    $list_id = $this->list_id();

    ?>
    <div id="mepr-mailpoet" class="mepr-autoresponder-config">
      <input type="checkbox" name="meprmailpoet_enabled" id="meprmailpoet_enabled" <?php checked($this->is_enabled()); ?> />
      <label for="meprmailpoet_enabled"><?php _e('Enable MailPoet', 'memberpress-mailpoet'); ?></label>
    </div>
    <?php if($this->mailpoet_active()): ?>
      <div id="mailpoet_hidden_area" class="mepr-options-sub-pane">
        <div id="meprmailpoet-options">
          <div id="meprmailpoet-list-id">
            <label>
              <span><?php _e('MailPoet List:', 'memberpress-mailpoet'); ?></span>
              <select name="meprmailpoet_list_id" id="meprmailpoet_list_id" class="mepr-text-input form-field">
                <?php foreach($all_lists as $l): ?>
                  <option value="<?php echo $l['list_id']; ?>" <?php selected($l['list_id'], $list_id); ?>><?php echo $l['name']; ?></option>
                <?php endforeach; ?>
              </select>
            </label>
          </div>
          <br/>
          <div id="meprmailpoet-optin">
            <label>
              <input type="checkbox" name="meprmailpoet_optin" id="meprmailpoet_optin" <?php checked($this->is_optin_enabled()); ?> />
              <span><?php _e('Enable Opt-In Checkbox', 'memberpress-mailpoet'); ?></span>
            </label>
            <div>
              <span class="description">
                <?php _e('If checked, an opt-in checkbox will appear on all of your membership registration pages.', 'memberpress-mailpoet'); ?>
              </span>
            </div>
          </div>
          <div id="meprmailpoet-optin-text" class="mepr-hidden mepr-options-panel">
            <label><?php _e('Signup Checkbox Label:', 'memberpress-mailpoet'); ?>
              <input type="text" name="meprmailpoet_text" id="meprmailpoet_text" value="<?php echo $this->optin_text(); ?>" class="form-field" size="75" />
            </label>
            <div><span class="description"><?php _e('This is the text that will display on the signup page next to your mailing list opt-in checkbox.', 'memberpress-mailpoet'); ?></span></div>
          </div>
        </div>
      </div>
    <?php else: ?>
      <div id="mailpoet_hidden_area" class="mepr-options-sub-pane">
        <p><?php _e('MailPoet Plugin not found. You must install the MailPoet Plugin to use this integration.', 'memberpress-mailpoet'); ?></p>
      </div>
    <?php endif; ?>
    <?php
  }

  public function store_option_fields() {
    update_option('meprmailpoet_enabled',      (isset($_POST['meprmailpoet_enabled'])));
    update_option('meprmailpoet_list_id',      (isset($_POST['meprmailpoet_list_id']))?stripslashes($_POST['meprmailpoet_list_id']):false);
    update_option('meprmailpoet_optin',        (isset($_POST['meprmailpoet_optin'])));
    update_option('meprmailpoet_text',         (isset($_POST['meprmailpoet_text']))?stripslashes($_POST['meprmailpoet_text']):$this->optin_text());
  }

  public function display_signup_field() {
    $mepr_options = MeprOptions::fetch();
    $post = MeprUtils::get_current_post();
    $prd = MeprProduct::is_product_page($post);

    //If the per membership list is enabled, and the global list is disabled -- then we should be sure the member doesn't see this
    if($prd !== false) {
      $enabled = (bool)get_post_meta($prd->ID, '_meprmailpoet_list_override', true);

      if($enabled && $mepr_options->disable_global_autoresponder_list) { return; }
    }

    if($this->is_enabled() and $this->is_optin_enabled()) {
      $optin = (MeprUtils::is_post_request())?isset($_POST['meprmailpoet_opt_in']):$mepr_options->opt_in_checked_by_default;

      ?>
      <div class="mp-form-row">
        <div class="mepr-mailpoet-signup-field">
          <div id="mepr-mailpoet-checkbox">
            <input type="checkbox" name="meprmailpoet_opt_in" id="meprmailpoet_opt_in" class="mepr-form-checkbox" <?php checked($optin); ?> />
            <span class="mepr-mailpoet-message"><?php echo $this->optin_text(); ?></span>
          </div>
          <div id="mepr-mailpoet-privacy">
            <small>
              <?php _e('We Respect Your Privacy', 'memberpress-mailpoet'); ?>
            </small>
          </div>
        </div>
      </div>
      <?php
     }
  }

  public function process_signup($txn) {
    $mepr_options = MeprOptions::fetch();

    $prd = $txn->product();
    $usr = $txn->user();

    $enabled = (bool)get_post_meta($prd->ID, '_meprmailpoet_list_override', true);

    //If the per membership list is enabled, and the global list is disabled -- then we should be sure the member doesn't get added
    if(!$this->is_enabled() || ($enabled && $mepr_options->disable_global_autoresponder_list)) { return; }

    if(!$this->is_optin_enabled() or ($this->is_optin_enabled() and isset($_POST['meprmailpoet_opt_in']))) {
      $this->add_subscriber($usr, $this->list_id());
    }
  }

  public function process_status_changes($obj, $sub_status = false) {
    if(!$this->is_enabled()) { return; }

    if($obj instanceof MeprTransaction && $sub_status !== false && $sub_status == MeprSubscription::$active_str) {
      return; //This is an expiring transaction which is part of an active subscription, so don't remove the user from the list
    }

    $user = new MeprUser($obj->user_id);

    //Member is active so let's not remove them
    if(in_array($obj->product_id, $user->active_product_subscriptions('ids', true))) {
      $this->maybe_add_subscriber($obj, $user);
    }
    else {
      $this->maybe_remove_subscriber($obj, $user);
    }
  }

  public function maybe_add_subscriber($obj, $user) {
    $enabled = (bool)get_post_meta($obj->product_id, '_meprmailpoet_list_override', true);
    $list_id = get_post_meta($obj->product_id, '_meprmailpoet_list_override_id', true);

    if($enabled && !empty($list_id) && $this->is_enabled()) {
      //If status is -1 (unsubscribed), then the user has explicitly asked for no more contact so don't re-add them
      $exists = $this->email_exists($user->user_email, true);

      if(!$exists || ($exists && $exists['status'] >= 0 && !$this->exists_in_list($exists['user_id'], $list_id))) {
        return $this->add_subscriber($user, $list_id);
      }
    }

    return false;
  }

  public function maybe_remove_subscriber($obj, $user) {
    $enabled = (bool)get_post_meta($obj->product_id, '_meprmailpoet_list_override', true);
    $list_id = get_post_meta($obj->product_id, '_meprmailpoet_list_override_id', true);

    if($enabled && !empty($list_id) && $this->is_enabled()) {
      return $this->remove_subscriber($user, $list_id);
    }

    return false;
  }

  public function display_product_override($product) {
    if(!$this->is_enabled()) { return; }

    $all_lists = $this->get_lists();
    $override_list = (bool)get_post_meta($product->ID, '_meprmailpoet_list_override', true);
    $override_list_id = get_post_meta($product->ID, '_meprmailpoet_list_override_id', true);

    ?>
    <div id="mepr-mailpoet" class="mepr-product-adv-item">
      <input type="checkbox" name="meprmailpoet_list_override" id="meprmailpoet_list_override" <?php checked($override_list); ?> />
      <label for="meprmailpoet_list_override"><?php _e('MailPoet list for this Membership', 'memberpress-mailpoet'); ?></label>

      <?php MeprAppHelper::info_tooltip('meprmailpoet-list-override',
                                        __('Enable Membership MailPoet List', 'memberpress-mailpoet'),
                                        __('If this is set the member will be added to this list when their payment is completed for this membership. If the member cancels or you refund their subscription, they will be removed from the list automatically.', 'memberpress-mailpoet'));
      ?>

      <div id="meprmailpoet_override_area" class="mepr-hidden product-options-panel">
        <label><?php _e('MailPoet List: ', 'memberpress-mailpoet'); ?></label>
        <select name="meprmailpoet_list_override_id" id="meprmailpoet_list_override_id" class="mepr-text-input form-field">
          <?php foreach($all_lists as $l): ?>
            <option value="<?php echo $l['list_id']; ?>" <?php selected($l['list_id'], $override_list_id); ?>><?php echo $l['name']; ?></option>
          <?php endforeach; ?>
        </select>
      </div>
    </div>
    <?php
  }

  public function save_product_override($product) {
    if(!$this->is_enabled()) { return; }

    if(isset($_POST['meprmailpoet_list_override'])) {
      update_post_meta($product->ID, '_meprmailpoet_list_override', true);
      update_post_meta($product->ID, '_meprmailpoet_list_override_id', stripslashes($_POST['meprmailpoet_list_override_id']));
    }
    else {
      update_post_meta($product->ID, '_meprmailpoet_list_override', false);
    }
  }

  public function get_lists() {
    if(!$this->mailpoet_active()) { return array(); }

    //This will return an array of results with the name and list_id of each mailing list
    $model_list = WYSIJA::get('list', 'model');
    return $model_list->get(array('name', 'list_id'), array('is_enabled' => 1));
  }

  public function email_exists($email, $add_status = false) {
    $model_user = WYSIJA::get('user', 'model');
    $model_user->getFormat = ARRAY_A;

    $exists = $model_user->getOne(array('user_id', 'status'), array('email' => $email));

    if($exists) {
      if($add_status) {
        return $exists;
      }
      else {
        return $exists['user_id'];
      }
    }

    return false;
  }

  //user_id is NOT a WP_User ID, it's MailPoet user id
  public function exists_in_list($user_id, $list_id) {
    global $wpdb;

    $exists_in_list = $wpdb->get_var("SELECT COUNT(*) FROM {$wpdb->prefix}wysija_user_list WHERE user_id = {$user_id} AND list_id = {$list_id} AND unsub_date <= 0");

    return ($exists_in_list > 0);
  }

  public function add_subscriber(MeprUser $contact, $list_id) {
    $user_data = array( 'email'     => $contact->user_email,
                        'firstname' => $contact->first_name,
                        'lastname'  => $contact->last_name );

    $data_subscriber = array( 'user'      => $user_data,
                              'user_list' => array('list_ids' => array($list_id)) );

    $helper_user = WYSIJA::get('user', 'helper');
    $helper_user->addSubscriber($data_subscriber);

    return true;
  }

  public function remove_subscriber(MeprUser $contact, $list_id) {
    if(($user_id = $this->email_exists($contact->user_email)) !== false) {
      $helper_user = WYSIJA::get('user', 'helper');
      $helper_user->removeFromLists(array($list_id), array($user_id));

      return true;
    }

    return false;
  }

  // I realize these are more like model methods
  // but we want everything centralized here people
  private function is_enabled() {
    return ($this->mailpoet_active() && get_option('meprmailpoet_enabled', false));
  }

  private function mailpoet_active() {
    return class_exists('WYSIJA');
  }

  private function list_id() {
    return get_option('meprmailpoet_list_id', false);
  }

  private function is_optin_enabled() {
    return get_option('meprmailpoet_optin', true);
  }

  private function optin_text() {
    $default = sprintf(__('Sign Up for the %s Newsletter', 'memberpress-mailpoet'), get_option('blogname'));
    return get_option('meprmailpoet_text', $default);
  }
} //END CLASS
